function varargout = AnnotationTool(varargin)
% ANNOTATIONTOOL M-file for AnnotationTool.fig
%
%     This M-file implements an annotation tool developed at 
%     University of Bonn for the eTRIMS Image Database. It is 
%     a MATLAB-based image annotation tool for the LabelMe MATLAB 
%     Toolbox developed at MIT.
% 
%     The tool is described in:
%         Annotation Tool for the LabelMe Toolbox
%         http://www.ipb.uni-bonn.de/~filip/AnnotationTool/
%         06-08-22-annotation-tool.pdf
%         August 2006
%
%     This program is free software; you can redistribute it and/or modify it.
%     It is distributed in the hope that it will be useful, but WITHOUT ANY 
%     WARRANTY; without even the implied warranty of MERCHANTABILITY or 
%     FITNESS FOR A PARTICULAR PURPOSE.
%
%     If you use this software for research purposes, please acknowledge
%     its use in resulting publication.
%
%     This program was developed by Filip Korc (filip.korc@uni-bonn.de) 
%     at the Department of Photogrammetry, University of Bonn, Bonn, Germany.
%
%     ANNOTATIONTOOL, by itself, creates a new ANNOTATIONTOOL or raises the existing
%     singleton*.
% 
%     H = ANNOTATIONTOOL returns the handle to a new ANNOTATIONTOOL or the handle to
%     the existing singleton*.
% 
%     ANNOTATIONTOOL('CALLBACK',hObject,eventData,handles,...) calls the local
%     function named CALLBACK in ANNOTATIONTOOL.M with the given input arguments.
% 
%     ANNOTATIONTOOL('Property','Value',...) creates a new ANNOTATIONTOOL or raises the
%     existing singleton*.  Starting from the left, property value pairs are
%     applied to the GUI before AnnotationTool_OpeningFunction gets called.  An
%     unrecognized property name or invalid value makes property application
%     stop.  All inputs are passed to AnnotationTool_OpeningFcn via varargin.
% 
%     *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%     instance to run (singleton)".
%
% Last Modified by GUIDE v2.5 16-Aug-2006 15:17:21

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @AnnotationTool_OpeningFcn, ...
                   'gui_OutputFcn',  @AnnotationTool_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before AnnotationTool is made visible.
function AnnotationTool_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to AnnotationTool (see VARARGIN)

% Choose default command line output for AnnotationTool
handles.output = hObject;

% flag: there are annotations to save
handles.MODIFIED = 0;

% image and annotation data paths
handles.HOMEIMAGES = deblank(readtextfile('home-images.txt'));
handles.HOMEANNOTATIONS = deblank(readtextfile('home-annotations.txt'));

% define colors
handles.colors = 'rgbcmyw';

% load object classes from a file to the popupmenu
tab = readtextfile('object-classes.txt');
c{1,1} = '';
for i=1:size(tab,1),
    c(i+1,1) = {deblank( tab(i,:) )};
end
sort(c); c{end+1,1} = 'other...';
set(handles.popup_object_name, 'String',c);

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes AnnotationTool wait for user response (see UIRESUME)
% uiwait(handles.figure1);

% --- Outputs from this function are returned to the command line.
function varargout = AnnotationTool_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in btn_open.
function btn_open_Callback(hObject, eventdata, handles)
% hObject    handle to btn_open (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Verzeichnis picken
directory = uigetdir(handles.HOMEIMAGES,'Select Image Directory');

if directory,

    % nur den letzten Teil des Strings extrahieren
    k=1;
    while true
       [t{k}, directory] = strtok(directory, '\');
       if isempty(t{k}),  break;  end
       k=k+1;
    end
    handles.directory = t{k-1};
    handles.files = dir([handles.HOMEIMAGES,'\', handles.directory]);
    handles.filenames = {};
    for i=3:length(handles.files),
        handles.filenames{end+1}=handles.files(i).name;
    end;
    set(handles.listbox_filenames,'String',handles.filenames);
    set(handles.listbox_filenames, 'Value',1);    
    % Sichern der handles-struktur
    guidata(hObject, handles);
    
    % enable btn
    set(handles.listbox_filenames,'Enable','on');
    
end

% --- Executes on selection change in listbox_filenames.
function listbox_filenames_Callback(hObject, eventdata, handles)
% hObject    handle to listbox_filenames (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns listbox_filenames contents as cell array
%        contents{get(hObject,'Value')} returns selected item from listbox_filenames

% select one annotation file from one of the folders:
% filename = fullfile( handles.directory,  handles.filenames{get(handles.listbox_filenames , 'value')});
% % read the image and annotation struct:
% filename
% [annotation, img] = LMread(filename, handles.HOMEIMAGES);
% % plot the annotations
% LMplot(annotation, img)

hold off;

% check for unsaved annotations
button_save_Callback(hObject, eventdata, handles)

% flag: there are annotations to save
handles.MODIFIED = 0;

handles.IMAGEfilename = ...
    handles.filenames{get(handles.listbox_filenames , 'value')};

handles.Image = imread([handles.HOMEIMAGES, '\', ...
                     handles.directory, '\', ...
                     handles.IMAGEfilename]);
                 
[pathstr, name, ext, versn] = fileparts(handles.IMAGEfilename);

handles.XMLfilename = [name,'.xml'];

if exist([handles.HOMEANNOTATIONS,'\',...
              handles.directory,'\',...
              handles.XMLfilename]),
          
    v = loadXML([handles.HOMEANNOTATIONS,'\',...
              handles.directory,'\',...
              handles.XMLfilename]);
    handles.annotation = v.annotation;
    [h, handles.class] = LMplot(v.annotation, handles.Image);
    axis('tight'), axis on
    set(handles.AnnotationListbox, 'String', handles.class);
    
    % read & display the source of the image
    if isfield(handles.annotation, 'sourceImage')
        s = get(handles.popup_image_source, 'String');
        r=[];
        for i = 1 : size(s,1), 
            r = [ r; strcmp( handles.annotation.sourceImage, ...
                s{i,1} ) ]; 
        end
        if find(r)
            set(handles.popup_image_source,'Value',find(r));
        else
            set(handles.popup_image_source,'Value',size(s,1));
        end
    end
    
else
    
    %fprintf('No annotation file available.\n\n');
    
    % create annotation struct
    handles.annotation = [];
    handles.annotation.filename = handles.IMAGEfilename;
    handles.annotation.folder = handles.directory;
    handles.annotation.sourceImage = '';
    
    %handles.annotation.object = []; %do not create!!!
    
    handles.class = {};
    
    image(handles.Image); axis('equal'); axis('tight'), hold on
    set(handles.AnnotationListbox, 'String', '');
    
    % clear the popup menus
    set(handles.popup_image_source,'Value',1);
    set(handles.popup_object_name,'Value',1);
    set(handles.popupmenu_occlusion,'Value',1);
    set(handles.popupmenu_representativeness,'Value',1);
    set(handles.popupmenu_uncertainty,'Value',1);

end;

% enable objects
set(handles.AnnotationListbox,'Enable','on');
set(handles.btn_zoom, 'Enable','on');
set(handles.popup_image_source, 'Enable','on');
set(handles.btn_annotate, 'Enable','on');
set(handles.popup_object_name, 'Enable','on');
set(handles.popup_image_source, 'Enable','on');

set(handles.AnnotationListbox, 'Value',1);                    

% Sichern der handles-struktur
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function listbox_filenames_CreateFcn(hObject, eventdata, handles)
% hObject    handle to listbox_filenames (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- Executes on selection change in AnnotationListbox.
function AnnotationListbox_Callback(hObject, eventdata, handles)
% hObject    handle to AnnotationListbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns AnnotationListbox contents as cell array
%        contents{get(hObject,'Value')} returns selected item from AnnotationListbox
%contents = get(hObject,'String')
i = get(hObject,'Value');
image(handles.Image);
%i = contents{get(hObject,'Value')}
%class{n} = handles.annotation.object(i).name; % get object name
[X,Y] = getLMpolygon(handles.annotation.object(i).polygon);

plot([X; X(1)],[Y; Y(1)], 'LineWidth', 4, 'color', [0 0 0]); hold on
h = plot([X; X(1)],[Y; Y(1)], 'LineWidth', 2, 'color', ...
          [rand, rand, rand]   );
      %handles.colors(mod(sum(double(handles.class{i})),7)+1)
hold on
axis on

% Sichern der handles-struktur
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function AnnotationListbox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to AnnotationListbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: listbox controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor', ...
        get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on button press in btn_add.
function btn_add_Callback(hObject, eventdata, handles)
% hObject    handle to btn_add (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

cont_annot = get(handles.popup_object_name,'String');
cont_occlusion = get(handles.popupmenu_occlusion,'String');
cont_representativeness = get(handles.popupmenu_representativeness,'String');
cont_uncertainty = get(handles.popupmenu_uncertainty,'String');
cont_source = get(handles.popup_image_source,'String');

if (strcmp(cont_annot{get(handles.popup_object_name,'Value')},'')==0) & ...
    (strcmp(cont_occlusion{get(handles.popupmenu_occlusion,'Value')},'')==0) & ...
    (strcmp(cont_representativeness{get(handles.popupmenu_representativeness,'Value')},'')==0) & ...
    (strcmp(cont_uncertainty{get(handles.popupmenu_uncertainty,'Value')},'')==0) & ...
    (strcmp(cont_source{get(handles.popup_image_source,'Value')},'')==0),    
    
    if ~isfield(handles,'login')
        % annotation login
        %prompt={'Enter your name:','Enter the place of annotation:'};
        prompt={'Enter your name:'};
        name='Annotation Login';
        numlines=1;
        defaultanswer={'Zimmermann@IPB'};
        cell_aux = inputdlg(prompt,name,numlines,defaultanswer);
        
        % check the login
        if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
            handles.login = cell_aux{1};
        end
    end
    
    if isfield(handles,'login')

        if isfield(handles.annotation,'object')
            i = length(handles.annotation.object);
        else
            i=0;
           % handles.annotation.object = [1];
        end;

        % object name
        handles.annotation.object(1,i+1).name = handles.currObject.name;
        handles.currObject.name = '';
        %cont_annot{get(handles.popup_object_name,'Value')};

        % degree of occlusion
        handles.annotation.object(1,i+1).occlusion = ...
            str2double( ...
            cont_occlusion{get(handles.popupmenu_occlusion,'Value')} );

        % representativeness of the annotated object with respect to the object
        % category
        handles.annotation.object(1,i+1).representativeness = ...
            str2double( cont_representativeness{ ...
            get(handles.popupmenu_representativeness,'Value') } );

        % uncertainty (accuracy) of the annotation [pixels]
        handles.annotation.object(1,i+1).uncertainty = ...
            str2double( cont_uncertainty{ ...
            get(handles.popupmenu_uncertainty,'Value')});

        % aux. tags
        handles.annotation.object(1,i+1).deleted = 0;
        handles.annotation.object(1,i+1).verified = 0;
        handles.annotation.object(1,i+1).date = date;
        %handles.annotation.object(1,i+1).polygon.username = 'AnnotationTool';
        handles.annotation.object(1,i+1).sourceAnnotation = handles.login;

        % polygon coordinates
        for l=1:size(handles.currObject.xy,2),
            handles.annotation.object(1,i+1).polygon.pt(l,1).x = ...
                num2str(handles.currObject.xy(1,l));
                
            handles.annotation.object(1,i+1).polygon.pt(l,1).y = ...
                num2str(handles.currObject.xy(2,l));
        end
        
        handles.currObject.xy = [];

        handles.class{end+1} = handles.annotation.object(1,i+1).name;
        set(handles.AnnotationListbox, 'String', handles.class);

        % clear the popup menus
        set(handles.popup_object_name,'Value',1);
        set(handles.popupmenu_occlusion,'Value',1);
        set(handles.popupmenu_representativeness,'Value',1);
        set(handles.popupmenu_uncertainty,'Value',1);
        
        % FLAGS:
        % annotations to save
        handles.MODIFIED = 1;
        % finished closed polygon
        handles.currObject.labeled = 0;
        
        % enable objects
        set(handles.button_save,'Enable','on');
        set(handles.btn_delete, 'Enable','on');

        % disable objects
        set(handles.popupmenu_occlusion,'Enable','off');
        set(handles.popupmenu_representativeness,'Enable','off');
        set(handles.popupmenu_uncertainty,'Enable','off');
        set(handles.btn_add,'Enable','off');

    elseif ~isempty(cell_aux)
        msgbox('Please fill in the login information.', 'Login')
    end
    
else
    msgbox('Please fill in the required information.')
end;

% update handles-structure
guidata(hObject, handles)


function popup_object_name_Callback(hObject, eventdata, handles)
% hObject    handle to popup_object_name (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of popup_object_name as text
%        str2double(get(hObject,'String')) returns contents of popup_object_name as a double

content = get(handles.popup_object_name,'String');
 if ~strcmp( content{end}, ...
        content{get(handles.popup_object_name,'Value')} );

    handles.currObject.name = ...
        content{get(handles.popup_object_name,'Value')};
 else
    cell_aux = inputdlg('Object Class: ','New Annotation');
    
    % check the returned string
    if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
        handles.currObject.name = cell_aux{1};
    else
        set(handles.popup_object_name,'Value',1);
    end
    
end

% update handles
guidata(hObject, handles) 

% --- Executes during object creation, after setting all properties.
function popup_object_name_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_object_name (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

function popup_image_source_Callback(hObject, eventdata, handles)
% hObject    handle to popup_image_source (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of popup_image_source as text
%        str2double(get(hObject,'String')) returns contents of popup_image_source as a double

content = get(handles.popup_image_source,'String');
if ~strcmp( content{end}, ...
        content{get(handles.popup_image_source,'Value')} );

    handles.annotation.sourceImage = ...
        content{get(handles.popup_image_source,'Value')};
    handles.MODIFIED = 1;
    set(handles.button_save,'Enable','on');

else
    cell_aux = inputdlg('Image source:','Current Image');
    
    % check the returned string
    if ~isempty(cell_aux) && ~strcmp(cell_aux{1},''),
        handles.annotation.sourceImage = cell_aux{1};
        handles.MODIFIED = 1;
        set(handles.button_save,'Enable','on');
    else
        handles.annotation.sourceImage = '';
        set(handles.popup_image_source,'Value',1);
        if ~isempty(cell_aux)
            handles.MODIFIED = 1;
            set(handles.button_save,'Enable','on');
        end
    end    
end

% update handles
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function popup_image_source_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popup_image_source (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- Executes on button press in savebutton.
function button_save_Callback(hObject, eventdata, handles)
% hObject    handle to savebutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if (handles.MODIFIED == 1),
    if ~strcmp(handles.annotation.sourceImage,'')
    
        if( strcmp( ...
                questdlg( 'Do you want to save the annotations?', ...
                'Confirm Save','Yes','No','Yes' ), ...
                'Yes' )),

            v.annotation = handles.annotation;
            meldung = mkdir( handles.HOMEANNOTATIONS, handles.directory );
            writeXML([handles.HOMEANNOTATIONS,'\',...
                      handles.directory,'\',...
                      handles.XMLfilename],...
                      v);
            clear('v');

            % disable btns
            set(handles.button_save,'Enable','off');
            set(handles.btn_delete, 'Enable','off');
            
            % flag: annotations to save
            handles.MODIFIED = 0;
            
        end;
    else
        msgbox('Please specify the Image Source.','Login')
    end
end;

% flag: there are annotations to save
handles.MODIFIED = 0;

% update handles
guidata(hObject, handles);


% --- Executes on selection change in popupmenu_occlusion.
function popupmenu_occlusion_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_occlusion (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_occlusion contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_occlusion

% --- Executes during object creation, after setting all properties.
function popupmenu_occlusion_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_occlusion (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on selection change in popupmenu_representativeness.
function popupmenu_representativeness_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_representativeness (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_representativeness contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_representativeness


% --- Executes during object creation, after setting all properties.
function popupmenu_representativeness_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_representativeness (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on selection change in popupmenu_uncertainty.
function popupmenu_uncertainty_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu_uncertainty (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu_uncertainty contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu_uncertainty

if isfield(handles, 'currObject') && handles.currObject.labeled
    
    % uncertainty (accuracy) of the annotation [pixels]
    contents = get(handles.popupmenu_uncertainty,'String'); hold on

    % uncertainty tool offset
    x_offset = sum(handles.currObject.xy(1,:))/size(handles.currObject.xy,2);
    y_offset = sum(handles.currObject.xy(2,:))/size(handles.currObject.xy,2);

    for i = 2:size(contents),
        plot_circle( [x_offset; y_offset], str2double(contents{i}), 'g')
    end
    plot_circle( [x_offset; y_offset], ...
        str2double(contents{get(handles.popupmenu_uncertainty,'Value')}), ...
        'r')
else
    msgbox('You have to label an object first!')
end
    

% --- Executes during object creation, after setting all properties.
function popupmenu_uncertainty_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu_uncertainty (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end

% --- aux. function that plots a circle of a given color
function plot_circle(OFFSET, SIZE, color)

SIZE = SIZE/2;
%OFFSET = 51;

t = 0:0.01:2*pi;

x = cos(t); 
y = sin(t);

x = x*SIZE+OFFSET(1); 
y = y*SIZE+OFFSET(2);

plot(x,y, color)


% --- Executes during object creation, after setting all properties.
function figure1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


% --- Executes on button press in btn_annotate.
function btn_annotate_Callback(hObject, eventdata, handles)
% hObject    handle to btn_annotate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[h, handles.class] = my_plot(handles.annotation, handles.Image);

% set flag (finished closed polygon)
handles.currObject.labeled = 0;

% disable objs
enable_var = disable_objects(ones(13,1), hObject, handles);

hold on; xy = []; n = 0;
% Loop, picking up the points.
disp('Left mouse button picks points.')
disp('Right mouse button closes the polygon.')
but = 1;
while but == 1
    [xi,yi,but] = ginput(1);
    if (but==1),
        plot(xi,yi,'ro')
        n = n+1;
        xy(:,n) = [xi;yi];
        if (n>1),
            %plot(xy(1,n-1:n),xy(2,n-1:n),'-r');
            plot(xy(1,n-1:n),xy(2,n-1:n), ...
                'LineWidth', 3, 'color', [0 0 0]); hold on
            plot(xy(1,n-1:n),xy(2,n-1:n), ...
                'LineWidth', 1, 'color', [rand, rand, rand]); 
            % handles.colors(mod(sum(double(handles.class{1})),7)+1)
        end;
    end;
    if (but==3),
        
        % enable objects
        enable_objects(enable_var, hObject, handles);
        
        if size(xy,2)>0
            
            % plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)],'-r');
            plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)], ...
                'LineWidth', 3, 'color', [0 0 0]); hold on
            plot([xy(1,n),xy(1,1)],[xy(2,n),xy(2,1)], ...
                'LineWidth', 1, 'color', [rand, rand, rand]); 

            % set flag (finished closed polygon)
            handles.currObject.labeled = 1;
            
            % enable objects
            enable_aux = [zeros(10,1); ones(3,1) ];
            enable_aux(9) = 1;
            enable_objects( enable_aux, hObject, handles);
            
        else
            
            % disable objects
            disable_aux = [zeros(10,1); ones(3,1) ];
            disable_aux(9) = 1;
            disable_objects( disable_aux, hObject, handles);
            
            % set flag (finished closed polygon)
            handles.currObject.labeled = 0;
        end
    end;        
end;

% save the polygon coordinates
handles.currObject.xy = xy;

% update handles-structure
guidata(hObject, handles)


% --- Executes on button press in btn_delete.
function btn_delete_Callback(hObject, eventdata, handles)
% hObject    handle to btn_delete (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if( strcmp( ...
        questdlg( 'Do you want to delete last annotated object?' ), ...
        'Yes' )),

    % delete last object
    handles.annotation.object = handles.annotation.object(1, 1:end-1);
    
    % update the listbox
    handles.class = handles.class(1:end-1);
    set(handles.AnnotationListbox, 'Value',1);
    set(handles.AnnotationListbox, 'String', handles.class);
        
    % disable the delete btn
    set(handles.btn_delete, 'Enable','off');
    
end

% update handles structure
guidata(hObject, handles);


% --- Executes on button press in btn_zoom.
function btn_zoom_Callback(hObject, eventdata, handles)
% hObject    handle to btn_zoom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if strcmp(get(handles.btn_annotate,'Enable'),'on')

    % toggle zoom status
    zoom
    
    % disable objs
    disable_aux = ones(13,1); 
    disable_aux(5) = 0;
    handles.enable_var = disable_objects( disable_aux, hObject, handles);
    
else
    % toggle zoom status
    zoom
    
    % enable objs
    enable_objects( handles.enable_var, hObject, handles);
    
end

% update handles structure
guidata(hObject, handles);

% --- 
function tab = readtextfile(filename)
% Read a text file into a matrix with one row per input line
% and with a fixed number of columns, set by the longest line.
% Each string is padded with NUL (ASCII 0) characters
%
% open the file for reading

ip = fopen(filename,'rt');          % 'rt' means read text
if (ip < 0)
    error('could not open file');   % just abort if error 
end;

% find length of longest line
max=0;                              % record length of longest string
cnt=0;                              % record number of strings
s = fgetl(ip);                      % get a line

while (ischar(s))                   % while not end of file
   cnt = cnt+1;
   if (length(s) > max)           % keep record of longest
       max = length(s);
   end;
   s = fgetl(ip);                  % get next line
end;

% rewind the file to the beginning
frewind(ip);

% create an empty matrix of appropriate size
tab=char(zeros(cnt,max));           % fill with ASCII zeros

% load the strings for real
cnt=0;
s = fgetl(ip);

while (ischar(s))
   cnt = cnt+1;
   tab(cnt,1:length(s)) = s;      % slot into table
    s = fgetl(ip);
end;

% close the file and return
fclose(ip);
return;


% --- 
function [h, class] = my_plot(annotation, img)
% Visualizes the polygons in an image.
colors = 'rgbcmyw';

% Draw image
image(img); axis('off'); hold on
% Draw each object (only non deleted ones)
h = []; class = [];
if isfield(annotation, 'object')
    Nobjects = length(annotation.object); n=0;
    for i = 1:Nobjects
        n = n+1;
        class{n} = annotation.object(i).name; % get object name
        [X,Y] = getLMpolygon(annotation.object(i).polygon);

        if isfield(annotation.object(i), 'confidence')
            LineWidth = round(annotation.object(i).confidence)
            LineWidth = min(max([2 LineWidth]), 8);
        else
            LineWidth = 4;
        end
        
        plot([X; X(1)],[Y; Y(1)], 'LineWidth', LineWidth, 'color', [0 0 0]); hold on
        h(n) = plot([X; X(1)],[Y; Y(1)], 'LineWidth', LineWidth/2, 'color', colors(mod(sum(double(class{n})),7)+1));
        hold on
    end

    if nargout == 0
        legend(h, class);
    end
end

axis on

% --- 
function v_out = disable_objects( v, hObject, handles )

v_out = v;

% disable btn
if v(1),
    if strcmp(get(handles.btn_open,'Enable'),'off')
        v_out(1) = 0;
    else
        set(handles.btn_open,'Enable','off');
    end
end

if v(2),
    if strcmp(get(handles.listbox_filenames,'Enable'),'off')
        v_out(2) = 0;
    else
        set(handles.listbox_filenames,'Enable','off');
    end
end

if v(3),
    if strcmp(get(handles.button_save,'Enable'),'off')
        v_out(3) = 0;
    else
        set(handles.button_save,'Enable','off');
    end
end

if v(4),
    if strcmp(get(handles.AnnotationListbox,'Enable'),'off')
        v_out(4) = 0;
    else
        set(handles.AnnotationListbox,'Enable','off');
    end
end

if v(5),
    if strcmp(get(handles.btn_zoom,'Enable'),'off')
        v_out(5) = 0;
    else
        set(handles.btn_zoom,'Enable','off');
    end
end

if v(6),
    if strcmp(get(handles.btn_delete,'Enable'),'off')
        v_out(6) = 0;
    else
        set(handles.btn_delete,'Enable','off');
    end
end

if v(7),
    if strcmp(get(handles.popup_image_source,'Enable'),'off')
        v_out(7) = 0;
    else
        set(handles.popup_image_source,'Enable','off');
    end
end

if v(8),
    if strcmp(get(handles.btn_annotate,'Enable'),'off')
        v_out(8) = 0;
    else
        set(handles.btn_annotate,'Enable','off');
    end
end

if v(9),
    if strcmp(get(handles.btn_add,'Enable'),'off')
        v_out(9) = 0;
    else
        set(handles.btn_add,'Enable','off');
    end
end

if v(10),
    if strcmp(get(handles.popup_object_name,'Enable'),'off')
        v_out(10) = 0;
    else
        set(handles.popup_object_name,'Enable','off');
    end
end
if v(11),
    if strcmp(get(handles.popupmenu_occlusion,'Enable'),'off')
        v_out(11) = 0;
    else
        set(handles.popupmenu_occlusion,'Enable','off');
    end
end
if v(12),
    if strcmp(get(handles.popupmenu_representativeness,'Enable'),'off')
        v_out(12) = 0;
    else
        set(handles.popupmenu_representativeness,'Enable','off');
    end
end
if v(13),
    if strcmp(get(handles.popupmenu_uncertainty,'Enable'),'off')
        v_out(13) = 0;
    else
        set(handles.popupmenu_uncertainty,'Enable','off');
    end
end

% update handles-structure
guidata(hObject, handles)


% --- 
function enable_objects( v, hObject, handles )

v_out = v;

% disable btn
if v(1),
    set(handles.btn_open,'Enable','on');
end

if v(2),
    set(handles.listbox_filenames,'Enable','on');
end

if v(3),
    set(handles.button_save,'Enable','on');
end

if v(4),
    set(handles.AnnotationListbox,'Enable','on');
end

if v(5),
    set(handles.btn_zoom,'Enable','on');
end

if v(6),
    set(handles.btn_delete,'Enable','on');
end

if v(7),
    set(handles.popup_image_source,'Enable','on');
end

if v(8),
    set(handles.btn_annotate,'Enable','on');
end

if v(9),
    set(handles.btn_add,'Enable','on');
end

if v(10),
    set(handles.popup_object_name,'Enable','on');
end

if v(11),
    set(handles.popupmenu_occlusion,'Enable','on');
end
if v(12),
    set(handles.popupmenu_representativeness,'Enable','on');
end
if v(13),
    set(handles.popupmenu_uncertainty,'Enable','on');
end

% update handles-structure
guidata(hObject, handles)
